# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from .samples import (
    SamplesResource,
    AsyncSamplesResource,
    SamplesResourceWithRawResponse,
    AsyncSamplesResourceWithRawResponse,
    SamplesResourceWithStreamingResponse,
    AsyncSamplesResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .evaluations import (
    EvaluationsResource,
    AsyncEvaluationsResource,
    EvaluationsResourceWithRawResponse,
    AsyncEvaluationsResourceWithRawResponse,
    EvaluationsResourceWithStreamingResponse,
    AsyncEvaluationsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .human_labels import (
    HumanLabelsResource,
    AsyncHumanLabelsResource,
    HumanLabelsResourceWithRawResponse,
    AsyncHumanLabelsResourceWithRawResponse,
    HumanLabelsResourceWithStreamingResponse,
    AsyncHumanLabelsResourceWithStreamingResponse,
)
from ...._base_client import make_request_options
from .generated_files import (
    GeneratedFilesResource,
    AsyncGeneratedFilesResource,
    GeneratedFilesResourceWithRawResponse,
    AsyncGeneratedFilesResourceWithRawResponse,
    GeneratedFilesResourceWithStreamingResponse,
    AsyncGeneratedFilesResourceWithStreamingResponse,
)
from ....types.dataframer import run_create_params
from ....types.dataframer.run_list_response import RunListResponse
from ....types.dataframer.run_create_response import RunCreateResponse
from ....types.dataframer.run_status_response import RunStatusResponse
from ....types.dataframer.run_update_response import RunUpdateResponse
from ....types.dataframer.run_retrieve_response import RunRetrieveResponse

__all__ = ["RunsResource", "AsyncRunsResource"]


class RunsResource(SyncAPIResource):
    @cached_property
    def evaluations(self) -> EvaluationsResource:
        return EvaluationsResource(self._client)

    @cached_property
    def generated_files(self) -> GeneratedFilesResource:
        return GeneratedFilesResource(self._client)

    @cached_property
    def human_labels(self) -> HumanLabelsResource:
        return HumanLabelsResource(self._client)

    @cached_property
    def samples(self) -> SamplesResource:
        return SamplesResource(self._client)

    @cached_property
    def with_raw_response(self) -> RunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return RunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return RunsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        number_of_samples: int,
        runtime_params: Dict[str, object],
        spec_version_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCreateResponse:
        """
        Create a generation run directly without using the /generate/ endpoint.

        **Note**: Most users should use `/api/dataframer/generate/` instead, which
        handles the full workflow of creating a run and submitting to the generation
        service.

        This endpoint creates a Run record in the database but does NOT submit it to the
        generation service. Use this only for advanced workflows or testing.

        Args:
          number_of_samples: Number of samples to generate

          runtime_params: Runtime parameters for generation (model, settings, etc.)

          spec_version_id: ID of the spec version to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/dataframer/runs/",
            body=maybe_transform(
                {
                    "number_of_samples": number_of_samples,
                    "runtime_params": runtime_params,
                    "spec_version_id": spec_version_id,
                },
                run_create_params.RunCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCreateResponse,
        )

    def retrieve(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunRetrieveResponse:
        """
        Get a specific run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunRetrieveResponse,
        )

    def update(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunUpdateResponse:
        """
        Update a run (only certain fields)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._put(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunListResponse:
        """
        Get all generation runs for the authenticated user's company, ordered by
        creation time (newest first).
        """
        return self._get(
            "/api/dataframer/runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunListResponse,
        )

    def delete(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a run and clean up associated S3 files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def status(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunStatusResponse:
        """
        Get run status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/api/dataframer/runs/{run_id}/status/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunStatusResponse,
        )


class AsyncRunsResource(AsyncAPIResource):
    @cached_property
    def evaluations(self) -> AsyncEvaluationsResource:
        return AsyncEvaluationsResource(self._client)

    @cached_property
    def generated_files(self) -> AsyncGeneratedFilesResource:
        return AsyncGeneratedFilesResource(self._client)

    @cached_property
    def human_labels(self) -> AsyncHumanLabelsResource:
        return AsyncHumanLabelsResource(self._client)

    @cached_property
    def samples(self) -> AsyncSamplesResource:
        return AsyncSamplesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncRunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return AsyncRunsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        number_of_samples: int,
        runtime_params: Dict[str, object],
        spec_version_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCreateResponse:
        """
        Create a generation run directly without using the /generate/ endpoint.

        **Note**: Most users should use `/api/dataframer/generate/` instead, which
        handles the full workflow of creating a run and submitting to the generation
        service.

        This endpoint creates a Run record in the database but does NOT submit it to the
        generation service. Use this only for advanced workflows or testing.

        Args:
          number_of_samples: Number of samples to generate

          runtime_params: Runtime parameters for generation (model, settings, etc.)

          spec_version_id: ID of the spec version to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/dataframer/runs/",
            body=await async_maybe_transform(
                {
                    "number_of_samples": number_of_samples,
                    "runtime_params": runtime_params,
                    "spec_version_id": spec_version_id,
                },
                run_create_params.RunCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCreateResponse,
        )

    async def retrieve(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunRetrieveResponse:
        """
        Get a specific run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunRetrieveResponse,
        )

    async def update(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunUpdateResponse:
        """
        Update a run (only certain fields)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._put(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunListResponse:
        """
        Get all generation runs for the authenticated user's company, ordered by
        creation time (newest first).
        """
        return await self._get(
            "/api/dataframer/runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunListResponse,
        )

    async def delete(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a run and clean up associated S3 files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/dataframer/runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def status(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunStatusResponse:
        """
        Get run status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/api/dataframer/runs/{run_id}/status/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunStatusResponse,
        )


class RunsResourceWithRawResponse:
    def __init__(self, runs: RunsResource) -> None:
        self._runs = runs

        self.create = to_raw_response_wrapper(
            runs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            runs.retrieve,
        )
        self.update = to_raw_response_wrapper(
            runs.update,
        )
        self.list = to_raw_response_wrapper(
            runs.list,
        )
        self.delete = to_raw_response_wrapper(
            runs.delete,
        )
        self.status = to_raw_response_wrapper(
            runs.status,
        )

    @cached_property
    def evaluations(self) -> EvaluationsResourceWithRawResponse:
        return EvaluationsResourceWithRawResponse(self._runs.evaluations)

    @cached_property
    def generated_files(self) -> GeneratedFilesResourceWithRawResponse:
        return GeneratedFilesResourceWithRawResponse(self._runs.generated_files)

    @cached_property
    def human_labels(self) -> HumanLabelsResourceWithRawResponse:
        return HumanLabelsResourceWithRawResponse(self._runs.human_labels)

    @cached_property
    def samples(self) -> SamplesResourceWithRawResponse:
        return SamplesResourceWithRawResponse(self._runs.samples)


class AsyncRunsResourceWithRawResponse:
    def __init__(self, runs: AsyncRunsResource) -> None:
        self._runs = runs

        self.create = async_to_raw_response_wrapper(
            runs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            runs.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            runs.update,
        )
        self.list = async_to_raw_response_wrapper(
            runs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            runs.delete,
        )
        self.status = async_to_raw_response_wrapper(
            runs.status,
        )

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResourceWithRawResponse:
        return AsyncEvaluationsResourceWithRawResponse(self._runs.evaluations)

    @cached_property
    def generated_files(self) -> AsyncGeneratedFilesResourceWithRawResponse:
        return AsyncGeneratedFilesResourceWithRawResponse(self._runs.generated_files)

    @cached_property
    def human_labels(self) -> AsyncHumanLabelsResourceWithRawResponse:
        return AsyncHumanLabelsResourceWithRawResponse(self._runs.human_labels)

    @cached_property
    def samples(self) -> AsyncSamplesResourceWithRawResponse:
        return AsyncSamplesResourceWithRawResponse(self._runs.samples)


class RunsResourceWithStreamingResponse:
    def __init__(self, runs: RunsResource) -> None:
        self._runs = runs

        self.create = to_streamed_response_wrapper(
            runs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            runs.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            runs.update,
        )
        self.list = to_streamed_response_wrapper(
            runs.list,
        )
        self.delete = to_streamed_response_wrapper(
            runs.delete,
        )
        self.status = to_streamed_response_wrapper(
            runs.status,
        )

    @cached_property
    def evaluations(self) -> EvaluationsResourceWithStreamingResponse:
        return EvaluationsResourceWithStreamingResponse(self._runs.evaluations)

    @cached_property
    def generated_files(self) -> GeneratedFilesResourceWithStreamingResponse:
        return GeneratedFilesResourceWithStreamingResponse(self._runs.generated_files)

    @cached_property
    def human_labels(self) -> HumanLabelsResourceWithStreamingResponse:
        return HumanLabelsResourceWithStreamingResponse(self._runs.human_labels)

    @cached_property
    def samples(self) -> SamplesResourceWithStreamingResponse:
        return SamplesResourceWithStreamingResponse(self._runs.samples)


class AsyncRunsResourceWithStreamingResponse:
    def __init__(self, runs: AsyncRunsResource) -> None:
        self._runs = runs

        self.create = async_to_streamed_response_wrapper(
            runs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            runs.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            runs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            runs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            runs.delete,
        )
        self.status = async_to_streamed_response_wrapper(
            runs.status,
        )

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResourceWithStreamingResponse:
        return AsyncEvaluationsResourceWithStreamingResponse(self._runs.evaluations)

    @cached_property
    def generated_files(self) -> AsyncGeneratedFilesResourceWithStreamingResponse:
        return AsyncGeneratedFilesResourceWithStreamingResponse(self._runs.generated_files)

    @cached_property
    def human_labels(self) -> AsyncHumanLabelsResourceWithStreamingResponse:
        return AsyncHumanLabelsResourceWithStreamingResponse(self._runs.human_labels)

    @cached_property
    def samples(self) -> AsyncSamplesResourceWithStreamingResponse:
        return AsyncSamplesResourceWithStreamingResponse(self._runs.samples)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SampleListParams"]


class SampleListParams(TypedDict, total=False):
    limit: int
    """Maximum samples to return (default: all, max: 1000)"""

    offset: int
    """Starting position in finish-time-ordered list (default: 0)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from dataframer import Dataframer, AsyncDataframer
from tests.utils import assert_matches_type
from dataframer.types.dataframer import (
    DatasetListResponse,
    DatasetUpdateResponse,
    DatasetRetrieveResponse,
    DatasetListFilesResponse,
    DatasetListFoldersResponse,
    DatasetCreateFromZipResponse,
    DatasetCreateWithFilesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Dataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.dataframer.datasets.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
        )
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Dataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.dataframer.datasets.with_raw_response.update(
                dataset_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert dataset is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert dataset is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Dataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.dataframer.datasets.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_from_zip(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        )
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_from_zip_with_all_params(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
            description="description",
        )
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_from_zip(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_from_zip(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_files(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        )
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_files_with_all_params(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
            config_json="config_json",
            csv_headers="csv_headers",
            description="description",
            file=b"raw file contents",
            files=[b"raw file contents"],
            folder_names=["string", "string"],
        )
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_with_files(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_with_files(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_files(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_files(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_files(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_files(self, client: Dataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.dataframer.datasets.with_raw_response.list_files(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_folders(self, client: Dataframer) -> None:
        dataset = client.dataframer.datasets.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_folders(self, client: Dataframer) -> None:
        response = client.dataframer.datasets.with_raw_response.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_folders(self, client: Dataframer) -> None:
        with client.dataframer.datasets.with_streaming_response.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_folders(self, client: Dataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.dataframer.datasets.with_raw_response.list_folders(
                "",
            )


class TestAsyncDatasets:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetRetrieveResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.dataframer.datasets.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
        )
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetUpdateResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncDataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.dataframer.datasets.with_raw_response.update(
                dataset_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert dataset is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert dataset is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncDataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.dataframer.datasets.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_from_zip(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        )
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_from_zip_with_all_params(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
            description="description",
        )
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_from_zip(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_from_zip(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.create_from_zip(
            name="name",
            zip_file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCreateFromZipResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_files(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        )
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_files_with_all_params(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
            config_json="config_json",
            csv_headers="csv_headers",
            description="description",
            file=b"raw file contents",
            files=[b"raw file contents"],
            folder_names=["string", "string"],
        )
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_with_files(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_with_files(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.create_with_files(
            dataset_type="SINGLE_FILE",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCreateWithFilesResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_files(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_files(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_files(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.list_files(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetListFilesResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_files(self, async_client: AsyncDataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.dataframer.datasets.with_raw_response.list_files(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_folders(self, async_client: AsyncDataframer) -> None:
        dataset = await async_client.dataframer.datasets.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_folders(self, async_client: AsyncDataframer) -> None:
        response = await async_client.dataframer.datasets.with_raw_response.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_folders(self, async_client: AsyncDataframer) -> None:
        async with async_client.dataframer.datasets.with_streaming_response.list_folders(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetListFoldersResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_folders(self, async_client: AsyncDataframer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.dataframer.datasets.with_raw_response.list_folders(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .spec_version import SpecVersion as SpecVersion
from .sample_classification import SampleClassification as SampleClassification
from .dataframer_list_models_response import DataframerListModelsResponse as DataframerListModelsResponse
from .dataframer_check_health_response import DataframerCheckHealthResponse as DataframerCheckHealthResponse
from .dataframer_list_historical_runs_response import (
    DataframerListHistoricalRunsResponse as DataframerListHistoricalRunsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RunRetrieveResponse"]


class RunRetrieveResponse(BaseModel):
    spec_version_id: str

    id: Optional[str] = None

    completed_at: Optional[datetime] = None

    created_at: Optional[datetime] = None

    created_by: Optional[int] = None

    created_by_name: Optional[str] = None

    dataset_id: Optional[str] = None

    dataset_name: Optional[str] = None

    dataset_type: Optional[str] = None

    duration_seconds: Optional[float] = None

    metrics_json: Optional[Dict[str, object]] = None

    number_of_samples: Optional[int] = None

    runtime_params: Optional[Dict[str, object]] = None

    spec_id: Optional[str] = None

    spec_name: Optional[str] = None

    spec_version: Optional[int] = None

    started_at: Optional[datetime] = None

    status: Optional[Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED", "CANCELED"]] = None

    status_display: Optional[str] = None

    trace: Optional[Dict[str, object]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["RunCreateParams"]


class RunCreateParams(TypedDict, total=False):
    number_of_samples: Required[int]
    """Number of samples to generate"""

    runtime_params: Required[Dict[str, object]]
    """Runtime parameters for generation (model, settings, etc.)"""

    spec_version_id: Required[str]
    """ID of the spec version to use"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.dataframer.red_team_run_list_response import RedTeamRunListResponse
from ...types.dataframer.red_team_run_create_response import RedTeamRunCreateResponse
from ...types.dataframer.red_team_run_retrieve_response import RedTeamRunRetrieveResponse
from ...types.dataframer.red_team_run_retrieve_status_response import RedTeamRunRetrieveStatusResponse

__all__ = ["RedTeamRunsResource", "AsyncRedTeamRunsResource"]


class RedTeamRunsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RedTeamRunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return RedTeamRunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RedTeamRunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return RedTeamRunsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunCreateResponse:
        """Create a new red team run and submit to data_gen backend"""
        return self._post(
            "/api/dataframer/red-team-runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunCreateResponse,
        )

    def retrieve(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunRetrieveResponse:
        """
        Retrieve a red team run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/api/dataframer/red-team-runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunListResponse:
        """List all red team runs for the user's company"""
        return self._get(
            "/api/dataframer/red-team-runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunListResponse,
        )

    def delete(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a red team run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/dataframer/red-team-runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve_status(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunRetrieveStatusResponse:
        """
        Get red team run status and sync with data_gen backend

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/api/dataframer/red-team-runs/{run_id}/status/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunRetrieveStatusResponse,
        )


class AsyncRedTeamRunsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRedTeamRunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncRedTeamRunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRedTeamRunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return AsyncRedTeamRunsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunCreateResponse:
        """Create a new red team run and submit to data_gen backend"""
        return await self._post(
            "/api/dataframer/red-team-runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunCreateResponse,
        )

    async def retrieve(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunRetrieveResponse:
        """
        Retrieve a red team run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/api/dataframer/red-team-runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunListResponse:
        """List all red team runs for the user's company"""
        return await self._get(
            "/api/dataframer/red-team-runs/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunListResponse,
        )

    async def delete(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a red team run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/dataframer/red-team-runs/{run_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve_status(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedTeamRunRetrieveStatusResponse:
        """
        Get red team run status and sync with data_gen backend

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/api/dataframer/red-team-runs/{run_id}/status/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedTeamRunRetrieveStatusResponse,
        )


class RedTeamRunsResourceWithRawResponse:
    def __init__(self, red_team_runs: RedTeamRunsResource) -> None:
        self._red_team_runs = red_team_runs

        self.create = to_raw_response_wrapper(
            red_team_runs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            red_team_runs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            red_team_runs.list,
        )
        self.delete = to_raw_response_wrapper(
            red_team_runs.delete,
        )
        self.retrieve_status = to_raw_response_wrapper(
            red_team_runs.retrieve_status,
        )


class AsyncRedTeamRunsResourceWithRawResponse:
    def __init__(self, red_team_runs: AsyncRedTeamRunsResource) -> None:
        self._red_team_runs = red_team_runs

        self.create = async_to_raw_response_wrapper(
            red_team_runs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            red_team_runs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            red_team_runs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            red_team_runs.delete,
        )
        self.retrieve_status = async_to_raw_response_wrapper(
            red_team_runs.retrieve_status,
        )


class RedTeamRunsResourceWithStreamingResponse:
    def __init__(self, red_team_runs: RedTeamRunsResource) -> None:
        self._red_team_runs = red_team_runs

        self.create = to_streamed_response_wrapper(
            red_team_runs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            red_team_runs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            red_team_runs.list,
        )
        self.delete = to_streamed_response_wrapper(
            red_team_runs.delete,
        )
        self.retrieve_status = to_streamed_response_wrapper(
            red_team_runs.retrieve_status,
        )


class AsyncRedTeamRunsResourceWithStreamingResponse:
    def __init__(self, red_team_runs: AsyncRedTeamRunsResource) -> None:
        self._red_team_runs = red_team_runs

        self.create = async_to_streamed_response_wrapper(
            red_team_runs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            red_team_runs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            red_team_runs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            red_team_runs.delete,
        )
        self.retrieve_status = async_to_streamed_response_wrapper(
            red_team_runs.retrieve_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dataframer import (
    DataframerResource,
    AsyncDataframerResource,
    DataframerResourceWithRawResponse,
    AsyncDataframerResourceWithRawResponse,
    DataframerResourceWithStreamingResponse,
    AsyncDataframerResourceWithStreamingResponse,
)

__all__ = [
    "DataframerResource",
    "AsyncDataframerResource",
    "DataframerResourceWithRawResponse",
    "AsyncDataframerResourceWithRawResponse",
    "DataframerResourceWithStreamingResponse",
    "AsyncDataframerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GenerateCreateParams"]


class GenerateCreateParams(TypedDict, total=False):
    generation_model: Required[Literal["claude-3-5-sonnet-v2@20241022", "gpt-4o", "gpt-4o-mini", "o1", "o1-mini"]]
    """AI model to use for generation"""

    number_of_samples: Required[int]
    """Number of samples to generate"""

    spec_id: Required[str]
    """ID of the spec to use for generation"""

    enable_revisions: bool
    """Enable revision cycles"""

    evaluation_model: Literal["claude-3-5-sonnet-v2@20241022", "gpt-4o", "gpt-4o-mini"]
    """AI model for evaluation (optional)"""

    evaluation_thinking_budget: int
    """Thinking budget for evaluation model (tokens, short samples)"""

    generation_thinking_budget: int
    """Thinking budget for generation model (tokens)"""

    max_iterations: int
    """Max feedback iterations (short samples only)"""

    max_revision_cycles: int
    """Max revision cycles (long samples only)"""

    num_examples_in_prompt: int
    """Number of examples to include in prompt (short samples only)"""

    outline_model: Literal["claude-3-5-sonnet-v2@20241022", "gpt-4o", "gpt-4o-mini"]
    """AI model for outline generation (long samples only)"""

    outline_thinking_budget: int
    """Thinking budget for outline model (tokens, long samples)"""

    revision_model: Literal["claude-3-5-sonnet-v2@20241022", "gpt-4o", "gpt-4o-mini"]
    """AI model for revisions (long samples only)"""

    revision_thinking_budget: int
    """Thinking budget for revision model (tokens, long samples)"""

    sample_type: Literal["short", "long"]
    """Type of samples to generate"""

    spec_version_id: str
    """Specific version ID to use (optional, defaults to latest version)"""

    staged_generation: bool
    """Use staged generation approach (short samples only)"""

    use_historical_feedback: bool
    """Use historical feedback (short samples only)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SpecCreateParams"]


class SpecCreateParams(TypedDict, total=False):
    datasets_id: Required[str]
    """UUID of the dataset this spec is based on"""

    name: Required[str]
    """Unique name for the spec (within dataset and company)"""

    description: str
    """Optional description of the spec"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..._types import (
    Body,
    Omit,
    Query,
    Headers,
    NoneType,
    NotGiven,
    FileTypes,
    SequenceNotStr,
    omit,
    not_given,
)
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.dataframer import dataset_update_params, dataset_create_from_zip_params, dataset_create_with_files_params
from ...types.dataframer.dataset_list_response import DatasetListResponse
from ...types.dataframer.dataset_update_response import DatasetUpdateResponse
from ...types.dataframer.dataset_retrieve_response import DatasetRetrieveResponse
from ...types.dataframer.dataset_list_files_response import DatasetListFilesResponse
from ...types.dataframer.dataset_list_folders_response import DatasetListFoldersResponse
from ...types.dataframer.dataset_create_from_zip_response import DatasetCreateFromZipResponse
from ...types.dataframer.dataset_create_with_files_response import DatasetCreateWithFilesResponse

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetRetrieveResponse:
        """
        Retrieve detailed information about a specific dataset including all files and
        folders.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            f"/api/dataframer/datasets/{dataset_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetRetrieveResponse,
        )

    def update(
        self,
        dataset_id: str,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetUpdateResponse:
        """Update dataset name and description.

        **Note:** Only metadata can be updated.

        Dataset type and structure cannot be
        changed after creation.

        Args:
          description: New dataset description

          name: New dataset name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._put(
            f"/api/dataframer/datasets/{dataset_id}/",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                dataset_update_params.DatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListResponse:
        """
        Retrieve all datasets belonging to the authenticated user's company.

        Returns dataset metadata including:

        - Dataset type (SINGLE_FILE, MULTI_FILE, MULTI_FOLDER)
        - File and folder counts
        - Short sample compatibility status
        - Creation information
        """
        return self._get(
            "/api/dataframer/datasets/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    def delete(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a dataset and all associated files from storage.

        **Warning:** This action cannot be undone. All files will be permanently deleted
        from S3.

        **Restrictions:** Cannot delete a dataset that is referenced by any specs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/dataframer/datasets/{dataset_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def create_from_zip(
        self,
        *,
        name: str,
        zip_file: FileTypes,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateFromZipResponse:
        """Upload a ZIP file containing dataset files.

        The backend will automatically:

        1. Extract and analyze the ZIP structure
        2. Auto-detect dataset type (SINGLE_FILE/MULTI_FILE/MULTI_FOLDER)
        3. Validate file types and sizes
        4. Create dataset with proper folder structure

        **Supported ZIP Structures:**

        1. **SINGLE_FILE**: ZIP contains exactly one valid file

           - Supported: CSV, JSON, JSONL
           - Max size: 50MB

        2. **MULTI_FILE**: ZIP contains multiple files in root

           - Supported: TXT, MD, JSON, CSV, JSONL
           - Max: 1MB per file, 50MB total, 1000 files

        3. **MULTI_FOLDER**: ZIP contains multiple folders with files
           - Supported: TXT, MD, JSON, CSV, JSONL
           - Max: 1MB per file, 50MB total, 20 files per folder
           - Requires at least 2 folders

        **Benefits:**

        - Single upload operation
        - Automatic structure detection
        - No need to specify dataset_type or folder_names
        - Works for all dataset types

        Args:
          name: Dataset name (unique within company)

          zip_file: ZIP file containing dataset files

          description: Optional dataset description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "zip_file": zip_file,
                "description": description,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["zip_file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/dataframer/datasets/create-from-zip/",
            body=maybe_transform(body, dataset_create_from_zip_params.DatasetCreateFromZipParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateFromZipResponse,
        )

    def create_with_files(
        self,
        *,
        dataset_type: Literal["SINGLE_FILE", "MULTI_FILE", "MULTI_FOLDER"],
        name: str,
        config_json: str | Omit = omit,
        csv_headers: str | Omit = omit,
        description: str | Omit = omit,
        file: FileTypes | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        folder_names: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateWithFilesResponse:
        """Create a new dataset and upload files in a single atomic operation.

        Files are
        validated first, and dataset is only created if validation passes. **Empty
        datasets cannot be created with this endpoint.**

        **Dataset Types:**

        1. **SINGLE_FILE**: Upload one file

           - Supported types: CSV, JSON, JSONL only
           - Max size: 50MB
           - Send file as 'file' parameter
           - Optional: 'csv_headers' as JSON array for CSV files

        2. **MULTI_FILE**: Upload multiple individual files

           - Supported types: TXT, MD, JSON, CSV, JSONL
           - Max size: 1MB per file, 50MB total
           - Max 1000 files per dataset
           - Send files as 'files' array parameter

        3. **MULTI_FOLDER**: Upload files in folders
           - Supported types: TXT, MD, JSON, CSV, JSONL
           - Max size: 1MB per file, 50MB total
           - Minimum 2 folders required, each folder must have at least 1 file
           - Max 1000 files total, 20 files per folder
           - Send files as 'files' array parameter
           - Send corresponding folder names as 'folder_names' array (parallel arrays)

        **Request Format:**

        - All requests must be multipart/form-data
        - Dataset metadata: name, description, dataset_type, config_json
        - Files depend on dataset_type (see above)

        **Benefits:**

        - Atomic operation - either everything succeeds or nothing is created
        - Validation before dataset creation
        - No empty datasets
        - Single API call instead of two

        Args:
          dataset_type: Dataset type: SINGLE_FILE, MULTI_FILE, or MULTI_FOLDER

          name: Dataset name (unique within company)

          config_json: Optional configuration JSON as string

          csv_headers: Optional CSV headers as JSON array for SINGLE_FILE CSV files

          description: Optional dataset description

          file: Single file for SINGLE_FILE dataset type

          files: Multiple files for MULTI_FILE or MULTI_FOLDER dataset types. Each file will be
              sent as a separate form field named 'files'. For MULTI_FOLDER: minimum 2 files
              required (at least 1 file per folder, across at least 2 folders).

          folder_names: Folder names for MULTI_FOLDER (parallel array with files). Each folder name is
              sent as a separate 'folder_names' form field. Minimum 2 unique folder names
              required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "dataset_type": dataset_type,
                "name": name,
                "config_json": config_json,
                "csv_headers": csv_headers,
                "description": description,
                "file": file,
                "files": files,
                "folder_names": folder_names,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["file"], ["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/dataframer/datasets/create/",
            body=maybe_transform(body, dataset_create_with_files_params.DatasetCreateWithFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateWithFilesResponse,
        )

    def list_files(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListFilesResponse:
        """
        Get all files in a specific dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            f"/api/dataframer/datasets/{dataset_id}/files/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListFilesResponse,
        )

    def list_folders(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListFoldersResponse:
        """
        Get all folders in a specific dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            f"/api/dataframer/datasets/{dataset_id}/folders/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListFoldersResponse,
        )


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/aimonlabs/dataframer-python-sdk#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetRetrieveResponse:
        """
        Retrieve detailed information about a specific dataset including all files and
        folders.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            f"/api/dataframer/datasets/{dataset_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetRetrieveResponse,
        )

    async def update(
        self,
        dataset_id: str,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetUpdateResponse:
        """Update dataset name and description.

        **Note:** Only metadata can be updated.

        Dataset type and structure cannot be
        changed after creation.

        Args:
          description: New dataset description

          name: New dataset name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._put(
            f"/api/dataframer/datasets/{dataset_id}/",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                dataset_update_params.DatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListResponse:
        """
        Retrieve all datasets belonging to the authenticated user's company.

        Returns dataset metadata including:

        - Dataset type (SINGLE_FILE, MULTI_FILE, MULTI_FOLDER)
        - File and folder counts
        - Short sample compatibility status
        - Creation information
        """
        return await self._get(
            "/api/dataframer/datasets/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    async def delete(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a dataset and all associated files from storage.

        **Warning:** This action cannot be undone. All files will be permanently deleted
        from S3.

        **Restrictions:** Cannot delete a dataset that is referenced by any specs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/dataframer/datasets/{dataset_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def create_from_zip(
        self,
        *,
        name: str,
        zip_file: FileTypes,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateFromZipResponse:
        """Upload a ZIP file containing dataset files.

        The backend will automatically:

        1. Extract and analyze the ZIP structure
        2. Auto-detect dataset type (SINGLE_FILE/MULTI_FILE/MULTI_FOLDER)
        3. Validate file types and sizes
        4. Create dataset with proper folder structure

        **Supported ZIP Structures:**

        1. **SINGLE_FILE**: ZIP contains exactly one valid file

           - Supported: CSV, JSON, JSONL
           - Max size: 50MB

        2. **MULTI_FILE**: ZIP contains multiple files in root

           - Supported: TXT, MD, JSON, CSV, JSONL
           - Max: 1MB per file, 50MB total, 1000 files

        3. **MULTI_FOLDER**: ZIP contains multiple folders with files
           - Supported: TXT, MD, JSON, CSV, JSONL
           - Max: 1MB per file, 50MB total, 20 files per folder
           - Requires at least 2 folders

        **Benefits:**

        - Single upload operation
        - Automatic structure detection
        - No need to specify dataset_type or folder_names
        - Works for all dataset types

        Args:
          name: Dataset name (unique within company)

          zip_file: ZIP file containing dataset files

          description: Optional dataset description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "zip_file": zip_file,
                "description": description,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["zip_file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/dataframer/datasets/create-from-zip/",
            body=await async_maybe_transform(body, dataset_create_from_zip_params.DatasetCreateFromZipParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateFromZipResponse,
        )

    async def create_with_files(
        self,
        *,
        dataset_type: Literal["SINGLE_FILE", "MULTI_FILE", "MULTI_FOLDER"],
        name: str,
        config_json: str | Omit = omit,
        csv_headers: str | Omit = omit,
        description: str | Omit = omit,
        file: FileTypes | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        folder_names: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateWithFilesResponse:
        """Create a new dataset and upload files in a single atomic operation.

        Files are
        validated first, and dataset is only created if validation passes. **Empty
        datasets cannot be created with this endpoint.**

        **Dataset Types:**

        1. **SINGLE_FILE**: Upload one file

           - Supported types: CSV, JSON, JSONL only
           - Max size: 50MB
           - Send file as 'file' parameter
           - Optional: 'csv_headers' as JSON array for CSV files

        2. **MULTI_FILE**: Upload multiple individual files

           - Supported types: TXT, MD, JSON, CSV, JSONL
           - Max size: 1MB per file, 50MB total
           - Max 1000 files per dataset
           - Send files as 'files' array parameter

        3. **MULTI_FOLDER**: Upload files in folders
           - Supported types: TXT, MD, JSON, CSV, JSONL
           - Max size: 1MB per file, 50MB total
           - Minimum 2 folders required, each folder must have at least 1 file
           - Max 1000 files total, 20 files per folder
           - Send files as 'files' array parameter
           - Send corresponding folder names as 'folder_names' array (parallel arrays)

        **Request Format:**

        - All requests must be multipart/form-data
        - Dataset metadata: name, description, dataset_type, config_json
        - Files depend on dataset_type (see above)

        **Benefits:**

        - Atomic operation - either everything succeeds or nothing is created
        - Validation before dataset creation
        - No empty datasets
        - Single API call instead of two

        Args:
          dataset_type: Dataset type: SINGLE_FILE, MULTI_FILE, or MULTI_FOLDER

          name: Dataset name (unique within company)

          config_json: Optional configuration JSON as string

          csv_headers: Optional CSV headers as JSON array for SINGLE_FILE CSV files

          description: Optional dataset description

          file: Single file for SINGLE_FILE dataset type

          files: Multiple files for MULTI_FILE or MULTI_FOLDER dataset types. Each file will be
              sent as a separate form field named 'files'. For MULTI_FOLDER: minimum 2 files
              required (at least 1 file per folder, across at least 2 folders).

          folder_names: Folder names for MULTI_FOLDER (parallel array with files). Each folder name is
              sent as a separate 'folder_names' form field. Minimum 2 unique folder names
              required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "dataset_type": dataset_type,
                "name": name,
                "config_json": config_json,
                "csv_headers": csv_headers,
                "description": description,
                "file": file,
                "files": files,
                "folder_names": folder_names,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["file"], ["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/dataframer/datasets/create/",
            body=await async_maybe_transform(body, dataset_create_with_files_params.DatasetCreateWithFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateWithFilesResponse,
        )

    async def list_files(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListFilesResponse:
        """
        Get all files in a specific dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            f"/api/dataframer/datasets/{dataset_id}/files/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListFilesResponse,
        )

    async def list_folders(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListFoldersResponse:
        """
        Get all folders in a specific dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            f"/api/dataframer/datasets/{dataset_id}/folders/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListFoldersResponse,
        )


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.retrieve = to_raw_response_wrapper(
            datasets.retrieve,
        )
        self.update = to_raw_response_wrapper(
            datasets.update,
        )
        self.list = to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = to_raw_response_wrapper(
            datasets.delete,
        )
        self.create_from_zip = to_raw_response_wrapper(
            datasets.create_from_zip,
        )
        self.create_with_files = to_raw_response_wrapper(
            datasets.create_with_files,
        )
        self.list_files = to_raw_response_wrapper(
            datasets.list_files,
        )
        self.list_folders = to_raw_response_wrapper(
            datasets.list_folders,
        )


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.retrieve = async_to_raw_response_wrapper(
            datasets.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            datasets.update,
        )
        self.list = async_to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            datasets.delete,
        )
        self.create_from_zip = async_to_raw_response_wrapper(
            datasets.create_from_zip,
        )
        self.create_with_files = async_to_raw_response_wrapper(
            datasets.create_with_files,
        )
        self.list_files = async_to_raw_response_wrapper(
            datasets.list_files,
        )
        self.list_folders = async_to_raw_response_wrapper(
            datasets.list_folders,
        )


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.retrieve = to_streamed_response_wrapper(
            datasets.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            datasets.update,
        )
        self.list = to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = to_streamed_response_wrapper(
            datasets.delete,
        )
        self.create_from_zip = to_streamed_response_wrapper(
            datasets.create_from_zip,
        )
        self.create_with_files = to_streamed_response_wrapper(
            datasets.create_with_files,
        )
        self.list_files = to_streamed_response_wrapper(
            datasets.list_files,
        )
        self.list_folders = to_streamed_response_wrapper(
            datasets.list_folders,
        )


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.retrieve = async_to_streamed_response_wrapper(
            datasets.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            datasets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            datasets.delete,
        )
        self.create_from_zip = async_to_streamed_response_wrapper(
            datasets.create_from_zip,
        )
        self.create_with_files = async_to_streamed_response_wrapper(
            datasets.create_with_files,
        )
        self.list_files = async_to_streamed_response_wrapper(
            datasets.list_files,
        )
        self.list_folders = async_to_streamed_response_wrapper(
            datasets.list_folders,
        )
